@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: hacks.h 3144 2005-11-12 18:11:49Z mosu $
+   $Id: hacks.h 3358 2006-11-10 18:23:51Z mosu $
 
    some hacks that the author might want to use
 
@@ -33,12 +33,11 @@ using namespace std;
 #define ENGAGE_NO_VARIABLE_DATA "no_variable_data"
 #define ENGAGE_NO_DEFAULT_HEADER_VALUES "no_default_header_values"
 #define ENGAGE_FORCE_PASSTHROUGH_PACKETIZER "force_passthrough_packetizer"
-#define ENGAGE_AVC_USE_BFRAMES "avc_use_bframes"
 #define ENGAGE_WRITE_HEADERS_TWICE "write_headers_twice"
 #define ENGAGE_ALLOW_AVC_IN_VFW_MODE "allow_avc_in_vfw_mode"
 #define ENGAGE_KEEP_BITSTREAM_AR_INFO "keep_bitstream_ar_info"
 #define ENGAGE_USE_SIMPLE_BLOCK "use_simpleblock"
-#define ENGAGE_NEW_AAC_CODECID "new_aac_codecid"
+#define ENGAGE_OLD_AAC_CODECID "old_aac_codecid"
 
 void MTX_DLL_API engage_hacks(const string &hacks);
 bool MTX_DLL_API hack_engaged(const string &hack);
