@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_matroska.cpp 3154 2005-12-02 14:14:45Z mosu $
+   $Id: r_matroska.cpp 3342 2006-11-08 13:39:55Z mosu $
 
    Matroska reader
 
@@ -440,7 +440,7 @@ kax_reader_c::verify_tracks() {
             t->a_formattag = FOURCC('f', 'L', 'a', 'C');
 #else
             mxwarn(PFX "mkvmerge was not compiled with FLAC support. "
-                   "Ignoring track %u.\n", t->tnum);
+                   "Ignoring track " LLU ".\n", t->tnum);
             continue;
 #endif
           } else if (t->codec_id == MKV_A_TTA)
@@ -1045,6 +1045,7 @@ kax_reader_c::read_headers() {
             KaxVideoPixelCropTop *kv_pctop;
             KaxVideoPixelCropRight *kv_pcright;
             KaxVideoPixelCropBottom *kv_pcbottom;
+            KaxVideoStereoMode *kv_stereo_mode;
 
             kv_pwidth = FINDFIRST(ktvideo, KaxVideoPixelWidth);
             if (kv_pwidth != NULL) {
@@ -1118,6 +1119,13 @@ kax_reader_c::read_headers() {
                        track->v_pcbottom);
             }
 
+            kv_stereo_mode = FINDFIRST(ktvideo, KaxVideoStereoMode);
+            if (NULL != kv_stereo_mode) {
+              track->v_stereo_mode = (stereo_mode_e)uint64(*kv_stereo_mode);
+              mxverb(2, PFX "|   + Stereo mode: %d\n",
+                     (int)track->v_stereo_mode);
+            }
+
           }
 
           kcodecid = FINDFIRST(ktentry, KaxCodecID);
@@ -1353,6 +1361,8 @@ kax_reader_c::init_passthrough_packetizer(kax_track_t *t) {
          (t->v_pcright > 0) || (t->v_pcbottom > 0)))
       ptzr->set_video_pixel_cropping(t->v_pcleft, t->v_pctop,
                                      t->v_pcright, t->v_pcbottom);
+    if (STEREO_MODE_UNSPECIFIED == ptzr->ti.stereo_mode)
+      ptzr->set_stereo_mode(t->v_stereo_mode);
     if (ptzr->get_cue_creation() == CUE_STRATEGY_UNSPECIFIED)
       ptzr->set_cue_creation( CUE_STRATEGY_IFRAMES);
 
@@ -1505,6 +1515,8 @@ kax_reader_c::create_packetizer(int64_t tid) {
             PTZR(t->ptzr)->set_video_pixel_cropping(t->v_pcleft, t->v_pctop,
                                                     t->v_pcright,
                                                     t->v_pcbottom);
+          if (STEREO_MODE_UNSPECIFIED == PTZR(t->ptzr)->ti.stereo_mode)
+            PTZR(t->ptzr)->set_stereo_mode(t->v_stereo_mode);
         } else
           init_passthrough_packetizer(t);
 
@@ -2195,6 +2207,8 @@ kax_reader_c::identify() {
         if ((0 != tracks[i]->v_dwidth) && (0 != tracks[i]->v_dheight))
           info += mxsprintf("display_dimensions:" LLU "x" LLU " ",
                             tracks[i]->v_dwidth, tracks[i]->v_dheight);
+        if (STEREO_MODE_UNSPECIFIED != tracks[i]->v_stereo_mode)
+          info += mxsprintf("stereo_mode:%d ", (int)tracks[i]->v_stereo_mode);
         info += mxsprintf("default_track:%u ",
                           tracks[i]->default_track ? 1 : 0);
         info += "]";
