@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_ogm.cpp 3160 2005-12-06 16:17:01Z mosu $
+   $Id: r_ogm.cpp 3331 2006-10-25 14:03:51Z mosu $
 
    OGG media stream reader
 
@@ -22,7 +22,12 @@
 #include <ogg/ogg.h>
 #include <vorbis/codec.h>
 #if defined(HAVE_FLAC_FORMAT_H)
-#include <FLAC/stream_decoder.h>
+# include <FLAC/stream_decoder.h>
+# if !defined(FLAC_API_VERSION_CURRENT) || FLAC_API_VERSION_CURRENT < 8
+#  define LEGACY_FLAC
+# else
+#  undef LEGACY_FLAC
+# endif
 #endif
 #if defined(SYS_WINDOWS)
 #include <windows.h>
@@ -102,17 +107,21 @@ extract_vorbis_comments(const memory_cptr &mem) {
 static FLAC__StreamDecoderReadStatus
 fhe_read_cb(const FLAC__StreamDecoder *decoder,
             FLAC__byte buffer[],
+#ifdef LEGACY_FLAC
             unsigned *bytes,
+#else
+            size_t *bytes,
+#endif
             void *client_data) {
   flac_header_extractor_c *fhe;
   ogg_packet op;
 
   fhe = (flac_header_extractor_c *)client_data;
   if (fhe->done)
-    return FLAC__STREAM_DECODER_READ_STATUS_END_OF_STREAM;
+    return FLAC__STREAM_DECODER_READ_STATUS_ABORT;
   if (ogg_stream_packetout(&fhe->os, &op) != 1) {
     if (!fhe->read_page() || (ogg_stream_packetout(&fhe->os, &op) != 1))
-      return FLAC__STREAM_DECODER_READ_STATUS_END_OF_STREAM;
+      return FLAC__STREAM_DECODER_READ_STATUS_ABORT;
   }
 
   if (*bytes < op.bytes)
@@ -193,6 +202,7 @@ flac_header_extractor_c::flac_header_extractor_c(const string &file_name,
   decoder = FLAC__stream_decoder_new();
   if (decoder == NULL)
     mxerror(FPFX "FLAC__stream_decoder_new() failed.\n");
+#ifdef LEGACY_FLAC
   FLAC__stream_decoder_set_client_data(decoder, this);
   if (!FLAC__stream_decoder_set_read_callback(decoder, fhe_read_cb))
     mxerror(FPFX "Could not set the read callback.\n");
@@ -202,10 +212,16 @@ flac_header_extractor_c::flac_header_extractor_c(const string &file_name,
     mxerror(FPFX "Could not set the metadata callback.\n");
   if (!FLAC__stream_decoder_set_error_callback(decoder, fhe_error_cb))
     mxerror(FPFX "Could not set the error callback.\n");
+#endif
   if (!FLAC__stream_decoder_set_metadata_respond_all(decoder))
     mxerror(FPFX "Could not set metadata_respond_all.\n");
+#ifdef LEGACY_FLAC
   if (FLAC__stream_decoder_init(decoder) !=
       FLAC__STREAM_DECODER_SEARCH_FOR_METADATA)
+#else
+  if (FLAC__stream_decoder_init_stream(decoder, fhe_read_cb, NULL, NULL, NULL, NULL, fhe_write_cb, fhe_metadata_cb, fhe_error_cb, this) !=
+      FLAC__STREAM_DECODER_INIT_STATUS_OK)
+#endif
     mxerror(FPFX "Could not initialize the FLAC decoder.\n");
   ogg_sync_init(&oy);
 }
@@ -672,6 +688,7 @@ ogm_reader_c::handle_new_stream(ogg_page *og) {
       return;
 
     dmx->stype = OGM_STREAM_TYPE_VORBIS;
+    dmx->num_header_packets = 3;
     dmx->in_use = true;
 
     return;
@@ -682,6 +699,7 @@ ogm_reader_c::handle_new_stream(ogg_page *og) {
       return;
 
     dmx->stype = OGM_STREAM_TYPE_THEORA;
+    dmx->num_header_packets = 3;
     dmx->in_use = true;
 
     try {
@@ -732,7 +750,6 @@ ogm_reader_c::handle_new_stream(ogg_page *og) {
         return;
 
       dmx->stype = OGM_STREAM_TYPE_VIDEO;
-      dmx->native_mode = false;
       if (video_fps < 0)
         video_fps = 10000000.0 / (float)get_uint64_le(&sth->time_unit);
       dmx->in_use = true;
@@ -764,7 +781,6 @@ ogm_reader_c::handle_new_stream(ogg_page *og) {
         return;
       }
 
-      dmx->native_mode = false;
       dmx->in_use = true;
 
       return;
@@ -775,7 +791,6 @@ ogm_reader_c::handle_new_stream(ogg_page *og) {
         return;
 
       dmx->stype = OGM_STREAM_TYPE_TEXT;
-      dmx->native_mode = false;
       dmx->in_use = true;
 
       return;
@@ -860,7 +875,7 @@ ogm_reader_c::process_page(ogg_page *og) {
 #endif
 
     if (dmx->stype == OGM_STREAM_TYPE_THEORA) {
-      int frame_number, keyframe_number, non_keyframe_number;
+      int keyframe_number, non_keyframe_number;
       int64_t timecode, duration, bref;
 
       if ((0 == op.bytes) || (0 != (op.packet[0] & 0x80)))
@@ -868,9 +883,8 @@ ogm_reader_c::process_page(ogg_page *og) {
 
       keyframe_number = ogg_page_granulepos(og) >> dmx->theora.kfgshift;
       non_keyframe_number = ogg_page_granulepos(og) & dmx->theora.kfgshift;
-      frame_number = keyframe_number + non_keyframe_number;
 
-      timecode = (int64_t)(1000000000.0 * dmx->last_granulepos *
+      timecode = (int64_t)(1000000000.0 * dmx->units_processed *
                            dmx->theora.frd / dmx->theora.frn);
       duration = (int64_t)(1000000000.0 * dmx->theora.frd / dmx->theora.frn);
       bref = (keyframe_number != dmx->last_keyframe_number) &&
@@ -879,10 +893,7 @@ ogm_reader_c::process_page(ogg_page *og) {
                                                          false), timecode,
                                             duration, bref, VFT_NOBFRAME));
 
-      if (frame_number > dmx->last_granulepos)
-        dmx->last_granulepos = frame_number;
-      else
-        ++dmx->last_granulepos;
+      dmx->units_processed++;
 
       dmx->last_keyframe_number = keyframe_number;
 
@@ -892,8 +903,6 @@ ogm_reader_c::process_page(ogg_page *og) {
         return;
       }
 
-      last_granulepos_set = true;
-
       continue;
     }
 
@@ -1002,8 +1011,14 @@ ogm_reader_c::process_header_packets(ogm_demuxer_t *dmx) {
   }
 
   while (ogg_stream_packetout(&dmx->os, &op) == 1) {
-    if (((dmx->stype != OGM_STREAM_TYPE_VORBIS) || !dmx->native_mode) &&
-        ((op.packet[0] & 1) != 1)) {
+    bool is_header_packet;
+
+    if (OGM_STREAM_TYPE_THEORA == dmx->stype)
+      is_header_packet = ((0x80 <= op.packet[0]) && (0x82 >= op.packet[0]));
+    else
+      is_header_packet = op.packet[0] & 1;
+
+    if (!is_header_packet) {
       mxwarn("ogm_reader: Missing header/comment packets for stream %d in "
              "'%s'. This file is broken but should be muxed correctly. If "
              "not please contact the author Moritz Bunkus "
@@ -1018,10 +1033,7 @@ ogm_reader_c::process_header_packets(ogm_demuxer_t *dmx) {
     dmx->packet_data.push_back(memory_cptr(mem));
   }
 
-  if (dmx->stype == OGM_STREAM_TYPE_VORBIS) {
-    if (dmx->packet_data.size() == 3)
-      dmx->headers_read = true;
-  } else if (dmx->packet_data.size() == 2)
+  if (dmx->packet_data.size() == dmx->num_header_packets)
     dmx->headers_read = true;
 }
 
@@ -1152,7 +1164,8 @@ ogm_reader_c::identify() {
             sdemuxers[i]->stype == OGM_STREAM_TYPE_AC3 ||
             sdemuxers[i]->stype == OGM_STREAM_TYPE_AAC ||
             sdemuxers[i]->stype == OGM_STREAM_TYPE_FLAC) ? "audio" :
-           sdemuxers[i]->stype == OGM_STREAM_TYPE_VIDEO ? "video" :
+           (sdemuxers[i]->stype == OGM_STREAM_TYPE_VIDEO ||
+            sdemuxers[i]->stype == OGM_STREAM_TYPE_THEORA) ? "video" :
            sdemuxers[i]->stype == OGM_STREAM_TYPE_TEXT ? "subtitles" :
            "unknown",
            sdemuxers[i]->stype == OGM_STREAM_TYPE_VORBIS ? "Vorbis" :
@@ -1161,6 +1174,7 @@ ogm_reader_c::identify() {
            sdemuxers[i]->stype == OGM_STREAM_TYPE_AC3 ? "AC3" :
            sdemuxers[i]->stype == OGM_STREAM_TYPE_AAC ? "AAC" :
            sdemuxers[i]->stype == OGM_STREAM_TYPE_VIDEO ? fourcc :
+           sdemuxers[i]->stype == OGM_STREAM_TYPE_THEORA ? "Theora" :
            sdemuxers[i]->stype == OGM_STREAM_TYPE_TEXT ? "text" :
            sdemuxers[i]->stype == OGM_STREAM_TYPE_FLAC ? "FLAC" :
            "unknown",
@@ -1178,9 +1192,6 @@ ogm_reader_c::handle_stream_comments() {
   string title;
   bool charset_warning_printed;
 
-  if (identifying)
-    return;
-
   charset_warning_printed = false;
   cch = utf8_init(ti.chapter_charset);
 
