@@ -8,7 +8,7 @@
    or visit http://www.gnu.org/copyleft/gpl.html
 
    \file
-   \version $Id: mkvmerge.cpp 3213 2006-04-28 11:33:27Z mosu $
+   \version $Id: mkvmerge.cpp 3341 2006-11-07 15:11:49Z mosu $
 
    \author Written by Moritz Bunkus <moritz@bunkus.org>.
    \author Modified by Steve Lhomme <steve.lhomme@free.fr>.
@@ -231,6 +231,10 @@ set_usage() {
     "                           Explicitely set the display dimensions.\n"
     "  --cropping <TID:left,top,right,bottom>\n"
     "                           Sets the cropping parameters.\n"
+    "  --stereo-mode <TID:n|none|left|right|both>\n"
+    "                           Sets the stereo mode parameter. It can\n"
+    "                           either be a numer 0 - 3 or one of the\n"
+    "                           keywords 'none', 'right', 'left' or 'both'.\n"
     "\n Options that only apply to text subtitle tracks:\n"
     "  --sub-charset <TID:charset>\n"
     "                           Sets the charset the text subtitles are\n"
@@ -668,6 +672,45 @@ parse_cropping(const string &s,
   ti.pixel_crop_list[id] = crop;
 }
 
+/** \brief Parse the \c --stereo-mode argument
+
+   The argument must either be a number between 0 and 3 or
+   one of the keywords \c 'none', \c 'left', \c 'right' or \c 'both'.
+*/
+static void
+parse_stereo_mode(const string &s,
+                  track_info_c &ti) {
+  static const char * const keywords[] = {
+    "none", "right", "left", "both", NULL
+  };
+  static const char *errmsg =
+    _("Stereo mode parameter: not given in the form "
+      "<TID>:<n|keyword> where n is a number between 0 and 3 "
+      "or one of the keywords 'none', 'right', 'left', 'both' "
+      "(argument was '%s').\n");
+  int64_t i, id;
+  vector<string> v;
+
+  v = split(s, ":");
+  if (v.size() != 2)
+    mxerror(errmsg, s.c_str());
+
+  id = 0;
+  if (!parse_int(v[0], id))
+    mxerror(errmsg, s.c_str());
+
+  for (i = 0; NULL != keywords[i]; ++i)
+    if (v[1] == keywords[i]) {
+      ti.stereo_mode_list[id] = (stereo_mode_e)i;
+      return;
+    }
+
+  if (!parse_int(v[1], i) || (i < 0) || (i > STEREO_MODE_BOTH))
+    mxerror(errmsg, s.c_str());
+
+  ti.stereo_mode_list[id] = (stereo_mode_e)i;
+}
+
 /** \brief Parse the duration formats to \c --split
 
   This function is called by ::parse_split if the format specifies
@@ -1297,11 +1340,13 @@ parse_args(vector<string> args) {
   int i;
   vector<string>::const_iterator sit;
   string this_arg, next_arg;
-  bool no_next_arg;
+  bool no_next_arg, inputs_found;
   int64_t id;
   attachment_t attachment;
   mm_io_c *io;
 
+  inputs_found = false;
+
   ti = new track_info_c;
 
   handle_common_cli_args(args, "");
@@ -1592,6 +1637,8 @@ parse_args(vector<string> args) {
 
       sit++;
 
+      inputs_found = true;
+
     } else if (this_arg == "--global-tags") {
       if (no_next_arg)
         mxerror(_("'--global-tags' lacks the file name.\n"));
@@ -1599,6 +1646,8 @@ parse_args(vector<string> args) {
       parse_and_add_tags(next_arg);
       sit++;
 
+      inputs_found = true;
+
     } else if (this_arg == "--chapter-language") {
       if (no_next_arg)
         mxerror(_("'--chapter-language' lacks the language.\n"));
@@ -1665,6 +1714,8 @@ parse_args(vector<string> args) {
                                     NULL, &tags_from_cue_chapters);
       sit++;
 
+      inputs_found = true;
+
     } else if (this_arg == "--segmentinfo") {
       if (no_next_arg)
         mxerror(_("'--segmentinfo' lacks the file name.\n"));
@@ -1679,6 +1730,8 @@ parse_args(vector<string> args) {
 
       sit++;
 
+      inputs_found = true;
+
     } else if (this_arg == "--no-chapters") {
       ti->no_chapters = true;
 
@@ -1796,6 +1849,13 @@ parse_args(vector<string> args) {
       parse_cropping(next_arg, *ti);
       sit++;
 
+    } else if (this_arg == "--stereo-mode") {
+      if (no_next_arg)
+        mxerror(_("'--stereo-mode' lacks the crop parameters.\n"));
+
+      parse_stereo_mode(next_arg, *ti);
+      sit++;
+
     } else if ((this_arg == "-y") || (this_arg == "--sync")) {
       if (no_next_arg)
         mxerror(_("'%s' lacks the audio delay.\n"), this_arg.c_str());
@@ -1993,6 +2053,9 @@ parse_args(vector<string> args) {
     mxwarn(_("'--link' is only useful in combination with '--split'.\n"));
 
   delete ti;
+
+  if (!inputs_found && (0 == files.size()))
+    mxerror("No input files were given. No output will be created.\n");
 }
 
 /** \brief Initialize global variables
