@@ -8,7 +8,7 @@
    or visit http://www.gnu.org/copyleft/gpl.html
 
    \file
-   \version $Id: output_control.cpp 3140 2005-11-04 08:58:57Z mosu $
+   \version $Id: output_control.cpp 3194 2006-02-27 17:06:55Z mosu $
 
    \author Written by Moritz Bunkus <moritz@bunkus.org>.
    \author Modified by Steve Lhomme <steve.lhomme@free.fr>.
@@ -90,6 +90,7 @@
 #include "r_vobsub.h"
 #include "r_wav.h"
 #include "r_wavpack.h"
+#include "r_corepicture.h"
 #include "tag_common.h"
 #include "xml_element_mapping.h"
 
@@ -356,6 +357,8 @@ get_file_type(filelist_t &file) {
       type = FILE_TYPE_SSA;
     else if (vobsub_reader_c::probe_file(text_io, size))
       type = FILE_TYPE_VOBSUB;
+    else if (corepicture_reader_c::probe_file(text_io, size))
+      type = FILE_TYPE_COREPICTURE;
     else if (usf_reader_c::probe_file(text_io, size))
       type = FILE_TYPE_USF;
     else
@@ -928,22 +931,31 @@ check_append_mapping() {
 
     // And now try to connect the packetizers.
     result = src_ptzr->can_connect_to(dst_ptzr, error_message);
-    if (CAN_CONNECT_YES != result) {
+    if (CAN_CONNECT_MAYBE_CODECPRIVATE == result)
+      mxwarn("The track number " LLD " from the file '%s' can probably "
+             "not be appended correctly to the track number " LLD
+             " from the file '%s': %s Please make sure that "
+             "the resulting file plays correctly the whole time. "
+             "The author of this program will probably not give support "
+             "for playback issues with the resulting file.\n",
+             amap->src_track_id, files[amap->src_file_id].name.c_str(),
+             amap->dst_track_id, files[amap->dst_file_id].name.c_str(),
+             error_message.c_str());
+
+    else if (CAN_CONNECT_YES != result) {
       string reason(result == CAN_CONNECT_NO_FORMAT ?
                     "the formats do not match" :
                     result == CAN_CONNECT_NO_PARAMETERS ?
                     "the track parameters do not match" :
                     "of unknown reasons");
-
-      if (CAN_CONNECT_NO_PARAMETERS == result)
-        reason += mxsprintf(" (%s)", error_message.c_str());
-
-      mxerror("The track number " LLD " from the file '%s' cannot be appended "
-              "to the track number " LLD " from the file '%s' because %s.\n",
+      mxerror("The track number " LLD " from the file '%s' cannot be "
+              "appended to the track number " LLD " from the file '%s' "
+              "because %s.\n",
               amap->src_track_id, files[amap->src_file_id].name.c_str(),
               amap->dst_track_id, files[amap->dst_file_id].name.c_str(),
               reason.c_str());
     }
+
     src_ptzr->connect(dst_ptzr);
     dst_file->appended_to = true;
   }
@@ -1097,6 +1109,9 @@ create_readers() {
         case FILE_TYPE_USF:
           file->reader = new usf_reader_c(*file->ti);
           break;
+        case FILE_TYPE_COREPICTURE:
+          file->reader = new corepicture_reader_c(*file->ti);
+          break;
         case FILE_TYPE_WAV:
           file->reader = new wav_reader_c(*file->ti);
           break;
