@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: mmg.cpp 3170 2005-12-14 15:30:23Z mosu $
+   $Id: mmg.cpp 3358 2006-11-10 18:23:51Z mosu $
 
    main stuff
 
@@ -137,9 +137,9 @@ wxString cli_options[][2] = {
         "mode, e.g. when it is read from an AVI") },
   { wxT("--engage use_simpleblock"),
     wxT("Enable use of SimpleBlock instead of BlockGroup when possible.") },
-  { wxT("--engage new_aac_codecid"),
-    wxT("Use the new AAC codec ID ('A_AAC') instead of the "
-        "old ones (e.g. 'A_AAC/MPEG4/SBR').") },
+  { wxT("--engage old_aac_codecid"),
+    wxT("Use the old AAC codec IDs (e.g. 'A_AAC/MPEG4/SBR') instead of the "
+        "new one ('A_AAC').") },
   { wxT("--engage cow"),
     wxT("No help available.") },
   { wxT(""), wxT("") }
@@ -1459,6 +1459,12 @@ mmg_dialog::update_command_line() {
         clargs.Add(sid + wxT(":") + t->fourcc);
       }
 
+      if (!t->appending && (0 != t->stereo_mode)) {
+        clargs.Add(wxT("--stereo-mode"));
+        clargs.Add(wxString::Format(wxT("%s:%d"), sid.c_str(),
+                                    t->stereo_mode - 1));
+      }
+
       if (!t->appending && (t->compression.Length() > 0)) {
         clargs.Add(wxT("--compression"));
         clargs.Add(sid + wxT(":") + t->compression);
@@ -1606,6 +1612,11 @@ mmg_dialog::update_command_line() {
       clargs.Add(strip(opts[i]));
   }
 
+  if (settings_page->cb_always_use_simpleblock->IsChecked()) {
+    clargs.Add(wxT("--engage"));
+    clargs.Add(wxT("use_simpleblock"));
+  }
+
   for (i = args_start; i < clargs.Count(); i++) {
     if (clargs[i].Find(wxT(" ")) >= 0)
       cmdline += wxT(" \"") + shell_escape(clargs[i]) + wxT("\"");
