@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: p_aac.cpp 3214 2006-04-28 12:02:02Z mosu $
+   $Id: p_aac.cpp 3358 2006-11-10 18:23:51Z mosu $
 
    AAC output module
 
@@ -130,7 +130,7 @@ aac_packetizer_c::get_aac_packet(unsigned long *header,
 
 void
 aac_packetizer_c::set_headers() {
-  if (hack_engaged(ENGAGE_NEW_AAC_CODECID))
+  if (!hack_engaged(ENGAGE_OLD_AAC_CODECID))
     set_codec_id(MKV_A_AAC);
   else if (id == AAC_ID_MPEG4) {
     if (profile == AAC_PROFILE_MAIN)
@@ -162,7 +162,7 @@ aac_packetizer_c::set_headers() {
 
   if ((ti.private_data != NULL) && (ti.private_size > 0))
     set_codec_private(ti.private_data, ti.private_size);
-  else if (hack_engaged(ENGAGE_NEW_AAC_CODECID)) {
+  else if (!hack_engaged(ENGAGE_OLD_AAC_CODECID)) {
     unsigned char buffer[5];
     int length;
 
